@@ -34,6 +34,7 @@
 #include "library/gnome-keyring-result.h"
 #include "library/gnome-keyring-opcodes.h"
 
+#include <security/pam_appl.h>
 #include <security/pam_modules.h>
 
 #include <sys/types.h>
@@ -117,13 +118,14 @@ typedef int (*line_cb) (char *line, void *arg);
 static int
 foreach_line (char *lines, line_cb cb, void *arg)
 {
-	char *line;
+	char *line, *ctx;
 	int ret;
 	
 	assert (lines);
 	
 	/* Call cb for each line in the text block */
-	while ((line = strsep (&lines, "\n")) != NULL) {
+	for (line = strtok_r (lines, "\n", &ctx); line != NULL; 
+	     line = strtok_r (NULL, "\n", &ctx)) {
 		 ret = (cb) (line, arg);
 		 if (ret != PAM_SUCCESS)
 		 	return ret;
@@ -135,6 +137,9 @@ foreach_line (char *lines, line_cb cb, void *arg)
 static char*
 read_all (int fd)
 {
+	/* We only accept a max of 8K from the daemon */
+	#define MAX_LENGTH 8192
+	
 	char buf[256];
 	char *ret = NULL;
 	int r, len = 0;
@@ -161,7 +166,7 @@ read_all (int fd)
 			strncat (ret, buf, r);
 		}
 		
-		if (r == 0)
+		if (r == 0 || len > MAX_LENGTH)
 			break;
 	}
 	
